package com.aliyun.chatbot.channel;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.chatbot.channel.chatbot.ChatbotClient;
import com.aliyun.chatbot.channel.chatbot.ChatbotSSERequest;
import com.aliyun.chatbot.channel.feishu.FeishuAccessTokenClient;
import com.aliyun.chatbot.channel.feishu.FeishuChannelMsgSender;
import com.aliyun.chatbot.channel.feishu.FeishuChatContext;
import com.aliyun.chatbot.channel.feishu.FeishuInputMessage;
import com.aliyun.chatbot.channel.feishu.FeishuInputMessage.MessageContent;
import com.lark.oapi.core.utils.Jsons;
import com.lark.oapi.event.EventDispatcher;
import com.lark.oapi.service.im.ImService.P2MessageReceiveV1Handler;
import com.lark.oapi.service.im.v1.model.P2MessageReceiveV1;
import com.lark.oapi.ws.Client;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

@Slf4j
public class FeishuRobotChatApp {

    // 机器人id
    public static final String CHATBOT_INSTANCE_ID = "chatbot-cn-******";

    // 业务空间AgentKey，获取地址：https://alime.console.aliyun.com/?productCode=p_ics#/business_management/business_management
    public static final String CHATBOT_AGENT_KEY = "**************_p_beebot_public";

    //aliyun账号AK、SK，登录阿里云账号获取
    public static final String ALIYUN_ACCESS_KEY = "<aliyun AK>";
    public static final String ALIYUN_SECRET_KEY = "<aliyun SK>";

    // 飞书应用的 Client ID
    public static final String FEISHU_CLIENT_ID = "cli_xxxxxxxx";
    // 飞书应用的 Client Secret
    public static final String FEISHU_CLIENT_SECRET = "xxxxxxxxxxxxxx";

    public static void main(String[] args) {
        ChatbotClient chatbotClient = new ChatbotClient(CHATBOT_AGENT_KEY, ALIYUN_ACCESS_KEY, ALIYUN_SECRET_KEY);

        FeishuAccessTokenClient feishuAccessTokenClient = new FeishuAccessTokenClient(FEISHU_CLIENT_ID,
            FEISHU_CLIENT_SECRET);
        RobotMsgCallbackConsumer msgConsumer = new RobotMsgCallbackConsumer(chatbotClient, feishuAccessTokenClient);

        EventDispatcher EVENT_HANDLER = EventDispatcher.newBuilder("", "")  // 长连接不需要这两个参数，请保持空字符串
            .onP2MessageReceiveV1(msgConsumer).build();

        Client client = new Client.Builder(FEISHU_CLIENT_ID, FEISHU_CLIENT_SECRET)
            .eventHandler(EVENT_HANDLER)
            .build();
        client.start();

    }

    public static class RobotMsgCallbackConsumer extends P2MessageReceiveV1Handler {
        private final ChatbotClient chatbotClient;
        private final FeishuAccessTokenClient dingAkClient;

        public RobotMsgCallbackConsumer(ChatbotClient chatbotClient,
            FeishuAccessTokenClient tokenClient) {
            this.chatbotClient = chatbotClient;
            this.dingAkClient = tokenClient;
        }

        @Override
        public void handle(P2MessageReceiveV1 event) throws Exception {
            log.info("receive message by robot:{}", Jsons.DEFAULT.toJson(event.getEvent()));
            try {
                FeishuInputMessage msg = JSON.parseObject(Jsons.DEFAULT.toJson(event.getEvent()),
                    FeishuInputMessage.class);
                if (msg == null || msg.getMessage() == null || !"text".equals(msg.getMessage().getMessageType())) {
                    return;
                }
                String content = msg.getMessage().getContent();
                MessageContent messageContent = JSON.parseObject(content, MessageContent.class);
                if (messageContent == null || StringUtils.isBlank(messageContent.getText())) {
                    return;
                }
                //机器人接收消息内容
                FeishuChatContext context = new FeishuChatContext()
                    .setFeishuAccessTokenClient(dingAkClient)
                    .setFeishuInputMessage(msg);
                ChannelMsgSender sender = new FeishuChannelMsgSender(context);
                sender.preSengMsg();

                String sessionId = msg.getMessage().getChatId() + msg.getSender().getSenderId();

                // 调用大模型
                String utterance = messageContent.getText();
                if ("group".equals(msg.getMessage().getChatType())) {
                    utterance = utterance.replaceAll("@_user_\\d+ ", "");
                }
                ChatbotSSERequest reqData = ChatbotSSERequest.build(CHATBOT_INSTANCE_ID,
                    utterance,
                    sessionId,
                    // TODO， 这里按场景选择使用OpenId还是UnionId
                    msg.getSender().getSenderId().getOpenId(),
                    // TODO， 获取用户名：https://open.feishu.cn/document/server-docs/contact-v3/user/get
                    msg.getSender().getSenderId().getOpenId());

                chatbotClient.invokeLlm(reqData, sender);

            } catch (Exception e) {
                log.error("receive message by robot error:" + e.getMessage(), e);
            }
        }
    }

}