package com.aliyun.chatbot.channel.feishu;

import java.util.HashMap;
import java.util.Map;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.chatbot.channel.utils.OkHttpUtils;
import lombok.extern.slf4j.Slf4j;

/**
 * @author kelin
 * @date 2024/6/7
 */
@Slf4j
public class FeishuChannelMsgSender extends ChannelMsgSender<FeishuChatContext> {

    public FeishuChannelMsgSender(FeishuChatContext context) {
        super(context);
    }

    @Override
    public void preSengMsg() {
        try {
            FeishuAccessTokenClient tokenClient = context.getFeishuAccessTokenClient();

            Map<String, String> map = new HashMap<>();
            map.put("type", "card_json");
            map.put("data",
                "{\"schema\":\"2.0\",\"header\":{\"title\":{\"content\":\"AI回复\",\"tag\":\"plain_text\"}},"
                    + "\"config\":{\"streaming_mode\":true,\"summary\":{\"content\":\"AI回复\"}},"
                    + "\"body\":{\"elements\":[{\"tag\":\"markdown\",\"content\":\"\","
                    + "\"element_id\":\"markdowncf687e2\"}]}}");
            String url = "https://open.feishu.cn/open-apis/cardkit/v1/cards/";
            String createCardResp = OkHttpUtils.postJsonParams(url, tokenClient.getAccessToken(),
                JSON.toJSONString(map));

            log.info("FeishuChannelMsgSender#preSengMsg CreateCard request:{}, resp:{}", JSON.toJSONString(map),
                createCardResp);

            String cardId = JSON.parseObject(createCardResp).getJSONObject("data").getString("card_id");

            context.setCardId(cardId);

            Map<String, String> sendCardReq = new HashMap<>();

            sendCardReq.put("content",
                "{\"type\":\"card\",\"data\":{\"card_id\":\"" + cardId + "\"}}");
            sendCardReq.put("msg_type", "interactive");
            sendCardReq.put("receive_id", context.getFeishuInputMessage().getSender().getSenderId().getOpenId());
            String sendCardUrl = "https://open.feishu.cn/open-apis/im/v1/messages?receive_id_type=open_id";
            if ("group".equals(context.getFeishuInputMessage().getMessage().getChatType())) {
                sendCardUrl = "https://open.feishu.cn/open-apis/im/v1/messages?receive_id_type=chat_id";
                sendCardReq.put("receive_id", context.getFeishuInputMessage().getMessage().getChatId());
            }
            String sendCardResp = OkHttpUtils.postJsonParams(sendCardUrl, tokenClient.getAccessToken(),
                JSON.toJSONString(sendCardReq));

            log.info("FeishuChannelMsgSender#preSengMsg sendCard request:{}, resp:{}",
                JSON.toJSONString(sendCardReq), sendCardResp);
        } catch (Exception e) {
            log.warn("FeishuChannelMsgSender#preSengMsg get exception, msg:{}", e.getMessage());
        }
    }

    @Override
    public void sendMsg(String reply, Boolean streamEnd) throws Exception {
        try {
            FeishuAccessTokenClient tokenClient = context.getFeishuAccessTokenClient();

            String cardId = context.getCardId();
            String url = "https://open.feishu.cn/open-apis/cardkit/v1/cards/" + cardId
                + "/elements/markdowncf687e2/content";
            Map<String, Object> param = new HashMap<>();
            param.put("content", reply);
            param.put("sequence", System.currentTimeMillis() - context.getStartTime());
            String response = OkHttpUtils.putJsonParams(url, tokenClient.getAccessToken(), JSON.toJSONString(param));

            log.info("FeishuChannelMsgSender#sendMsg get resp:{}", response);

            // TODO 引用文档输出


        } catch (Exception e) {
            log.error("FeishuChannelMsgSender#sendMsg get exception, msg:{}", e.getMessage());
        }
    }

    @Override
    public void postSendMsg() {
        log.info("CHAT_END================================================");

    }
}
