package com.aliyun.chatbot.channel.utils;

import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

import lombok.extern.slf4j.Slf4j;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Request.Builder;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;

/**
 * @author kelin
 * @date 2025/1/20
 */
@Slf4j
public class OkHttpUtils {
    private static final OkHttpClient okHttpClient = new OkHttpClient.Builder()
        .connectTimeout(60, TimeUnit.SECONDS)//设置连接超时时间
        .readTimeout(60, TimeUnit.SECONDS)//设置读取超时时间
        .build();

    public static String get(String url, Map<String, String> queries) {
        String responseBody = "";
        StringBuilder sb = new StringBuilder(url);
        if (queries != null && queries.keySet().size() > 0) {
            boolean firstFlag = true;
            for (Entry<String, String> entry : queries.entrySet()) {
                if (firstFlag) {
                    sb.append("?").append(entry.getKey()).append("=").append(entry.getValue());
                    firstFlag = false;
                } else {
                    sb.append("&").append(entry.getKey()).append("=").append(entry.getValue());
                }
            }

        }
        Request request = new Request.Builder()
            .url(sb.toString())
            .build();
        try (Response response = okHttpClient.newCall(request).execute()) {
            if (response.isSuccessful() && response.body() != null) {
                return response.body().string();
            }
        } catch (Exception e) {
            log.error("okhttp3 put error >> ", e);
        }
        return responseBody;
    }

    public static String postJsonParams(String url, String jsonParams) {
        return postJsonParams(url, null, jsonParams);
    }

    public static String postJsonParams(String url, String token, String jsonParams) {
        String responseBody = "";
        RequestBody requestBody = RequestBody.create(MediaType.parse("application/json; charset=utf-8"), jsonParams);
        Builder builder = new Builder()
            .url(url)
            .post(requestBody);
        if (StringUtils.isNotBlank(token)) {
            builder.addHeader("Authorization", "Bearer " + token);
        }
        Request request = builder
            .build();

        try (Response response = okHttpClient.newCall(request).execute()) {
            if (response.isSuccessful() && response.body() != null) {
                return response.body().string();
            } else {
                log.error(response.body().string());
            }
        } catch (Exception e) {
            log.error("okhttp3 postJsonParams error >> ", e);
        }
        return responseBody;
    }

    public static String putJsonParams(String url, String token, String jsonParams) {
        String responseBody = "";
        RequestBody requestBody = RequestBody.create(MediaType.parse("application/json; charset=utf-8"), jsonParams);
        Builder builder = new Builder()
            .url(url)
            .put(requestBody);
        if (StringUtils.isNotBlank(token)) {
            builder.addHeader("Authorization", "Bearer " + token);
        }
        Request request = builder
            .build();

        try (Response response = okHttpClient.newCall(request).execute()) {
            if (response.isSuccessful() && response.body() != null) {
                return response.body().string();
            } else {
                log.error(response.body().string());
            }
        } catch (Exception e) {
            log.error("okhttp3 putJsonParams error >> ", e);
        }
        return responseBody;
    }

}
